@@ -1,5 +1,8 @@
-from inspect import getargspec, getmembers, isclass, ismethod
-from util import _cfg
+from inspect import getargspec, getmembers, isclass, ismethod, isfunction
+
+import six
+
+from .util import _cfg
 
 __all__ = [
     'expose', 'transactional', 'accept_noncanonical', 'after_commit',
@@ -30,9 +33,10 @@ def expose(template=None,
                      directory.
     :param content_type: The content-type to use for this template.
     :param generic: A boolean which flags this as a "generic" controller,
-                    which uses generic functions based upon ``simplegeneric``
-                    generic functions.  Allows you to split a single
-                    controller into multiple paths based upon HTTP method.
+                    which uses generic functions based upon
+                    ``functools.singledispatch`` generic functions.  Allows you
+                    to split a single controller into multiple paths based upon
+                    HTTP method.
     '''
 
     if template == 'json':
@@ -74,7 +78,10 @@ def transactional(ignore_redirects=True):
 
     def deco(f):
         if isclass(f):
-            for meth in [m[1] for m in getmembers(f) if ismethod(m[1])]:
+            for meth in [
+                m[1] for m in getmembers(f)
+                if (isfunction if six.PY3 else ismethod)(m[1])
+            ]:
                 if getattr(meth, 'exposed', False):
                     _cfg(meth)['transactional'] = True
                     _cfg(meth)['transactional_ignore_redirects'] = _cfg(
