@@ -3,8 +3,8 @@ from inspect import getmembers
 
 from webob.exc import HTTPFound
 
-from util import iscontroller, _cfg
-from routing import lookup_controller
+from .util import iscontroller, _cfg
+from .routing import lookup_controller
 
 __all__ = [
     'PecanHook', 'TransactionHook', 'HookController',
@@ -29,18 +29,25 @@ def walk_controller(root_class, controller, hooks):
                 walk_controller(root_class, value, hooks)
 
 
-class HookController(object):
+class HookControllerMeta(type):
     '''
     A base class for controllers that would like to specify hooks on
     their controller methods. Simply create a list of hook objects
     called ``__hooks__`` as a member of the controller's namespace.
     '''
 
-    __hooks__ = []
+    def __init__(cls, name, bases, dict_):
+        walk_controller(cls, cls, dict_.get('__hooks__', []))
 
-    class __metaclass__(type):
-        def __init__(cls, name, bases, dict_):
-            walk_controller(cls, cls, dict_['__hooks__'])
+
+HookController = HookControllerMeta(
+    'HookController',
+    (object,),
+    {'__doc__': ("A base class for controllers that would like to specify "
+                 "hooks on their controller methods. Simply create a list "
+                 "of hook objects called ``__hooks__`` as a class attribute "
+                 "of your controller.")}
+)
 
 
 class PecanHook(object):
@@ -233,14 +240,14 @@ class RequestViewerHook(PecanHook):
     **blacklist**
 
     This key holds items that will be blacklisted based on ``url``. If
-    there is a need to ommit urls that start with `/javascript`, then this
+    there is a need to omit urls that start with `/javascript`, then this
     key would look like::
 
         'blacklist': ['/javascript']
 
     As many blacklisting items as needed can be contained in the list. The hook
     will verify that the url is not starting with items in this list to display
-    results, otherwise it will get ommited.
+    results, otherwise it will get omitted.
 
     .. note::
         This key should always use a ``list`` of items to use.
@@ -304,7 +311,7 @@ class RequestViewerHook(PecanHook):
                     value = getattr(state.request, request_info)
                 else:
                     value = value(self, state)
-            except Exception, e:
+            except Exception as e:
                 value = e
 
             terminal.append('%-12s - %s\n' % (request_info, value))
